/**********************************************************************
** Copyright (C) 2001 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** Licensees holding valid Qtopia Developer license may use this
** file in accordance with the Qtopia Developer License Agreement
** provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
** THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** email sales@trolltech.com for information about Qtopia License
** Agreements.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "qpsword.h"
#include <qpushbutton.h>
#include <qtabwidget.h>
#include "qswtextview.h"
#include <qcombobox.h>
#include <qspinbox.h>
#include <qpopupmenu.h>

#include <swmgr.h>
#include <versekey.h>
#include <markupfiltmgr.h>

#include <qlineedit.h>
/* 
 *  Constructs a Example which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
QPSword::QPSword( QWidget* parent,  const char* name, WFlags fl )
    : MainForm( parent, name, fl )
{
//    connect(quit, SIGNAL(clicked()), this, SLOT(goodBye()));
	mgr = new SWMgr(new MarkupFilterMgr(FMT_HTMLHREF));
	ldFrame->hide();
	bookIndexFrame->hide();
	currentDisplay = textDisplay;
	currentText = 0;
	currentLD = 0;
	SWModule *module = 0;
	for (ModMap::iterator it = mgr->Modules.begin(); it != mgr->Modules.end(); it++) {
		module = it->second;
		if (!strcmp(module->Type(),"Biblical Texts"))
			currentText = module;
		if (!strcmp(module->Type(), "Lexicons / Dictionaries"))
			currentLD = module;
	}

	SWConfig userConfig("swordprefs.conf");
	if (userConfig["layout"]["currentText"].length()) {
	    module = mgr->Modules[userConfig["layout"]["currentText"]];
	    if (module)
		    currentText = module;
	}
	if (userConfig["layout"]["currentLD"].length()) {
	    module = mgr->Modules[userConfig["layout"]["currentLD"]];
	    if (module)
		    currentLD = module;
	}


	refreshText();


	if (userConfig["layout"]["textFrame"].length()) {
		if (userConfig["layout"]["textFrame"] == "On")
			textFrame->show();
		else textFrame->hide();
	}

	if (userConfig["layout"]["ldFrame"].length()) {
		if (userConfig["layout"]["ldFrame"] == "On")
			ldFrame->show();
		else ldFrame->hide();
	}

	if (userConfig["layout"]["verseSelectFrame"].length()) {
		if (userConfig["layout"]["verseSelectFrame"] == "On")
			verseSelectFrame->show();
		else verseSelectFrame->hide();
	}

	if (userConfig["layout"]["bookIndexFrame"].length()) {
		if (userConfig["layout"]["bookIndexFrame"] == "On")
			bookIndexFrame->show();
		else bookIndexFrame->hide();
	}


	modsPopup = 0;
	connect( bookCB, SIGNAL( activated(const QString&) ), this, SLOT( refreshText() ) );
	connect( chapSB, SIGNAL( valueChanged ( int ) ), this, SLOT( refreshText() ) );
	connect( verseSB, SIGNAL( valueChanged ( int ) ), this, SLOT( refreshText() ) );
	connect( ldSelectEdit, SIGNAL( textChanged ( const QString& ) ), this, SLOT( refreshLD() ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
QPSword::~QPSword()
{
	SWConfig userConfig("swordprefs.conf");

	if (currentText)
		userConfig["layout"]["currentText"] = currentText->Name();
	if (currentLD)
		userConfig["layout"]["currentLD"] = currentLD->Name();

	userConfig["layout"]["textFrame"] = ((!textFrame->isHidden()) ? "On" : "Off");
	userConfig["layout"]["ldFrame"] = ((!ldFrame->isHidden()) ? "On" : "Off");
	userConfig["layout"]["verseSelectFrame"] = ((!verseSelectFrame->isHidden()) ? "On" : "Off");
	userConfig["layout"]["bookIndexFrame"] = ((!bookIndexFrame->isHidden()) ? "On" : "Off");

	userConfig.Save();

    // no need to delete child widgets, Qt does it all for us
	if (mgr)
		delete mgr;
}


void QPSword::refreshLD() {
	if (!currentLD) return;	// assert book is selected
	currentLD->SetKey((const char *)ldSelectEdit->text());
	QString text = "<html><body>";
	text += "<font color=\"#0000ff\"><small>" + QString(currentLD->KeyText()) + "</small></font>";
	text += QString::fromUtf8(((const char *)*currentLD));
	text += "</body></html>";
	ldDisplay->setText(text);
	// assume we want to see if we're refreshing
	if (ldFrame->isHidden())
		ldFrame->show();
}


void QPSword::refreshText() {
	if (!currentText) return;	// assert book is selected
	bool wholeChapter = false;

	QString keyText = "";
	if (((SWKey *)(*currentText))->getClass()->isAssignableFrom("VerseKey")) {
		keyText= bookCB->currentText() + " " + chapSB->text() + ":" + verseSB->text();
		if (!strcmp(currentText->Type(),"Biblical Texts"))
			wholeChapter = true;
	}
	else {
		keyText = "/book/index";
	}

	currentText->SetKey((const char *)keyText);

	if (wholeChapter) {	// display an entire chapter
		VerseKey *key = (VerseKey *)(SWKey *)(*currentText);
		int verse = key->Verse();
		int chapter = key->Chapter();
		int book = key->Book();
		key->Verse(1);
		QString text = "<html><body>";
		for (key->Verse(1); ((key->Chapter() == chapter) && (key->Book() == book)); (*currentText)++) {
			text += "<font color=\"#0000ff\"><small>" + QString::number(key->Verse()) + "</small></font>";
			if (key->Verse() == verse)
				text += "<font color=\"#00ff00\">";
			text += QString::fromUtf8(((const char *)*currentText)) + " ";
			if (key->Verse() == verse)
				text += "</font>";
		}
		text += "</body></html>";
		textDisplay->setText(text);
	}
	else {
		QString text = "<html><body>";
		text += "<font color=\"#0000ff\"><small>" + QString(currentText->KeyText()) + "</small></font>";
		text += QString::fromUtf8(((const char *)*currentText));
		text += "</body></html>";
		textDisplay->setText(text);
	}
	if (textFrame->isHidden())
		textFrame->show();
}

void QPSword::showLocator() {
	if (!currentText) return;	// assert book is selected

	if (((SWKey *)(*currentText))->getClass()->isAssignableFrom("VerseKey")) {
		if (verseSelectFrame->isHidden())
			verseSelectFrame->show();
		else	verseSelectFrame->hide();
	}
	else {
		if (bookIndexFrame->isHidden())
			bookIndexFrame->show();
		else	bookIndexFrame->hide();
	}
}


void QPSword::showLD() {
	if (ldFrame->isHidden())
		ldFrame->show();
	else	ldFrame->hide();
}


void QPSword::showBook() {
	if (textFrame->isHidden())
		textFrame->show();
	else	textFrame->hide();
}

void QPSword::showTextContextMenu() {
    QPopupMenu * m = new QPopupMenu( this );
    QPSword *app = (QPSword *)this->topLevelWidget();

    int id = 0;

    modsPopup = new QPopupMenu( this );
    SWModule *module = 0;
    for (ModMap::iterator it = mgr->Modules.begin(); it != mgr->Modules.end(); it++) {
	    module = it->second;
	    id = modsPopup->insertItem( module->Name(), app, SLOT( chooseModule(int) ) );
	    modsPopup->setItemChecked(id, ((module == currentText)||(module == currentLD)));
    }

	m->insertItem( tr( "Choose Module" ), modsPopup );

	optionsPopup = new QPopupMenu( this );
	id = optionsPopup->insertItem( "Footnotes", app, SLOT( toggleOption(int) ) );

	// TODO: make this dynamic
	const char *optVal = mgr->getGlobalOption ("Footnotes");
	if (optVal)
		optionsPopup->setItemChecked(id, (!strcmp(optVal, "On")));

	id = optionsPopup->insertItem( "Strong's Numbers", app, SLOT( toggleOption(int) ) );
	optVal = mgr->getGlobalOption ("Strong's Numbers");
	if (optVal)
		optionsPopup->setItemChecked(id, (!strcmp(optVal, "On")));

	m->insertItem( tr( "Module Options" ), optionsPopup );
    
    m->insertSeparator();
    m->insertItem( tr( "Lookup" ), app, SLOT( lookup() ) );
    m->insertSeparator();

    if (!app->ldFrame->isHidden()) {
        id = m->insertItem( tr( "Show Book" ), app, SLOT( showBook() ) );
        m->setItemChecked(id, !app->textFrame->isHidden());
        id = m->insertItem( tr( "Show Key Nav" ), app, SLOT( showLocator() ) );
        m->setItemChecked(id, ((!app->verseSelectFrame->isHidden())||(!app->bookIndexFrame->isHidden())));
    }
    if (!app->textFrame->isHidden()) {
        id = m->insertItem( tr( "Show Lex/Dict" ), app, SLOT( showLD() ) );
        m->setItemChecked(id, !app->ldFrame->isHidden());
    }
    m->popup( QCursor::pos() );
}

// close();
void QPSword::lookup() {

	if (!currentDisplay->hasSelectedText()) return;

	QString lookupText = currentDisplay->selectedText().stripWhiteSpace();
	char *ltext = 0;
	stdstr(&ltext, (const char *)lookupText);
	char *scan = 0;
	for (scan = ltext; *scan; scan++) {
		if (*scan == '<' || *scan == '>') *scan = ' ';
		if ((!isdigit(*scan)) && (*scan != ' '))
			break;
	}

	if (!*scan) {	// we a strongs number
		SWModule *strongsLex = 0;
		strongsLex = mgr->Modules[((bookCB->currentItem() < 39) ? "StrongsHebrew" : "StrongsGreek")];
		if (strongsLex)
			currentLD = strongsLex;
	}
	lookupText = ltext;
	lookupText = lookupText.stripWhiteSpace();
	ldSelectEdit->setText(lookupText);

	delete [] ltext;

	refreshLD();
}


void QPSword::chooseModule(int id) {
	QString modName = modsPopup->text(id);
	SWModule *module = mgr->Modules[(const char *)modName];
	if (!strcmp(module->Type(), "Lexicons / Dictionaries")) {
		currentLD = module;
		refreshLD();
	}
	else {
		currentText = module;
		refreshText();
	}
}


void QPSword::toggleOption(int id) {
	QString optionName = optionsPopup->text(id);

	const char *optVal = mgr->getGlobalOption((const char *)optionName);
//	qWarning( optVal );
	mgr->setGlobalOption ((const char *)optionName, ((!strcmp(optVal, "Off")) ? "On" : "Off"));
	refreshText();
}
